@@ -42,7 +42,7 @@ enum sample_compat {
 int sample_setup_maps(struct bpf_map **maps, const char *ifname);
 int __sample_init(int mask, int ifindex_from, int ifindex_to);
 void sample_teardown(void);
-int sample_run(int interval, void (*post_cb)(void *), void *ctx);
+int sample_run(unsigned int interval, void (*post_cb)(void *), void *ctx);
 bool sample_is_compat(enum sample_compat compat_value);
 bool sample_probe_cpumap_compat(void);
 bool sample_probe_xdp_load_bytes(void);
